/*=============================================================================
File:	common.h
Purpose:	implements the string_buffer definition

Special thanks to Taku Kudo <taku@chasen.org> for orginal implementation in
http://crfpp.sourceforge.net/

Simplifies only for readability and understandability
   Copyright(C) langine team
   email: nlptech360@gmail.com
   The license is under GNU GPL

Removes those never used codes in this project
=============================================================================*/

#ifndef DARTS_H_
#define DARTS_H_

namespace CRFPP {

namespace Darts {

	class DoubleArrayImpl
	{
		private:	
		struct unit_t
		{
			int   base;
			unsigned int check;
		};

		unit_t        *array_;
		size_t        size_;
		bool          no_delete_;

		public:
		explicit DoubleArrayImpl(): array_(0), size_(0), no_delete_(0) {}
		~DoubleArrayImpl() { clear(); }
		
		void set_result(int& x, int r, size_t) { x = r; }
		void set_array(void *ptr, size_t size = 0)
		{
			clear();
			array_ = reinterpret_cast<unit_t *>(ptr);
			no_delete_ = true;
			size_ = size;
		}

		void clear()
		{
			if (!no_delete_)
				delete [] array_;
			array_ = 0;			
			size_ = 0;
		}

		inline int exactMatchSearch(const char *key, size_t len = 0, size_t node_pos = 0)
		{
			if (!len) len = strlen(key);
			int result;
			set_result(result, -1, 0);
			register int  b = array_[node_pos].base;
			register  unsigned int p;
			for (register size_t i = 0; i < len; ++i)
			{
				p = b +(unsigned char)(key[i]) + 1;
				if (static_cast< unsigned int>(b) == array_[p].check)
					b = array_[p].base;
				else
					return result;
			}

			p = b;
			int n = array_[p].base;
			if (static_cast< unsigned int>(b) == array_[p].check && n < 0)
				set_result(result, -n-1, len);
			return result;
		}
	};
}
}
#endif
