/*=============================================================================
File:	feature_cache.cpp
Purpose:	speeds up the feature operation for cache

Special thanks to Taku Kudo <taku@chasen.org> for orginal implementation in
http://crfpp.sourceforge.net/

Simplifies only for readability and understandability
   Copyright(C) langine team
   email: nlptech360@gmail.com
   The license is under GNU GPL.

Removes the boundary features like EOS/BOS compared with the original version
=============================================================================*/

#include "feature_cache.h"

namespace CRFPP {

void FeatureCache::add(const std::vector<int> &f)
{
	int *p = feature_freelist_.alloc(f.size() + 1);
	std::copy(f.begin(), f.end(), p);
	p[f.size()] = -1;
	this->push_back(p);
}

void FeatureCache::shrink(std::map<int, int> *old2new)
{
	for (size_t i = 0; i < size(); ++i)
	{
		std::vector<int> newf;
		for (int *f = (*this)[i]; *f != -1; ++f)
		{
			std::map<int, int>::iterator it = old2new->find(*f);
			if (it != old2new->end()) newf.push_back(it->second);
		}
		newf.push_back(-1);
		std::copy(newf.begin(), newf.end(), (*this)[i]);
  }

  return;
}
}
