/*=============================================================================
File:	common.h
Purpose:	implements the string_buffer definition

Special thanks to Taku Kudo <taku@chasen.org> for orginal implementation in
http://crfpp.sourceforge.net/

Simplifies only for readability and understandability
   Copyright(C) langine team
   email: nlptech360@gmail.com
   The license is under GNU GPL

Removes those never used codes in this project
=============================================================================*/

#ifndef CRFPP_FEATURE_INDEX_H_
#define CRFPP_FEATURE_INDEX_H_

#include <vector>
#include "common.h"
#include "scoped_ptr.h"
#include "feature_cache.h"
#include "path.h"
#include "mmap.h"
#include "darts.h"

namespace CRFPP {

class TaggerImpl;

class FeatureIndex {
 protected:
  unsigned int        maxid_;
  double             *alpha_;
  float              *alpha_float_;
  double              cost_factor_;
  unsigned int        xsize_;
  unsigned int        max_xsize_;
  size_t              thread_num_;
  FeatureCache        feature_cache_;
  std::vector<char*>  unigram_templs_;
  std::vector<char*>  bigram_templs_;
  std::vector<char*>  y_;
  FreeList<char>      char_freelist_;
  scoped_array< FreeList<Path> > path_freelist_;
  scoped_array< FreeList<Node> > node_freelist_;

  virtual int getID(const char *) = 0;

  const char *get_index(char *&, size_t, const TaggerImpl &);
  bool apply_rule(string_buffer *,
                  char *,
                  size_t, const TaggerImpl &);

 public:
  static const unsigned int version;

  size_t size() const  { return maxid_; }
  size_t xsize() const { return xsize_; }
  size_t ysize() const { return y_.size(); }
  const char* y(size_t i) const { return y_[i]; }
  void   set_alpha(double *alpha) { alpha_ = alpha; }
  const float *alpha_float() { return const_cast<float *>(alpha_float_); }
  const double *alpha() const { return const_cast<double *>(alpha_); }
  char *strdup(const char *);
  void calcCost(Node *);
  void calcCost(Path *);

  bool buildFeatures(TaggerImpl *);
  void rebuildFeatures(TaggerImpl *);
  virtual bool open(const char*) = 0;
  virtual void clear() = 0;

  void init() {
    path_freelist_.reset(new FreeList<Path> [thread_num_]);
    node_freelist_.reset(new FreeList<Node> [thread_num_]);
    for (size_t i = 0; i < thread_num_; ++i) {
      path_freelist_[i].set_size(8192 * 16);
      node_freelist_[i].set_size(8192);
    }
  }

  explicit FeatureIndex(): maxid_(0), alpha_(0), alpha_float_(0),
                           cost_factor_(1.0), xsize_(0), max_xsize_(0),
                           thread_num_(1), char_freelist_(8192) {}
  virtual ~FeatureIndex() {}
};

class DecoderFeatureIndex: public FeatureIndex {
 private:
  Mmap <char> mmap_;
  Darts::DoubleArrayImpl da_;
  int getID(const char *);
 public:
  explicit DecoderFeatureIndex() { init(); }
  bool open(const char *);
  void clear();
};
}

#endif
