/*=============================================================================
File:	common.h
Purpose:	implements the string_buffer definition

Special thanks to Taku Kudo <taku@chasen.org> for orginal implementation in
http://crfpp.sourceforge.net/

Simplifies only for readability and understandability
   Copyright(C) langine team
   email: nlptech360@gmail.com
   The license is under GNU GPL

Removes the codes running on linux platform, who want to debug this project on
linux, please add these code first.
=============================================================================*/

#ifndef CRFPP_MMAP_H_
#define CRFPP_MMAP_H_

#ifdef WIN32
#include <windows.h>
#endif

namespace CRFPP{

template <class T> class Mmap{
private:
	T            *text;
	size_t       length;
	std::string  fileName;
	HANDLE hFile;
	HANDLE hMap;
public:
	T&       operator[](size_t n)       { return *(text + n); }
	const T& operator[](size_t n) const { return *(text + n); }
	T*       begin()           { return text; }
	const T* begin()    const  { return text; }
	T*       end()           { return text + size(); }
	const T* end()    const  { return text + size(); }
	size_t size()               { return length/sizeof(T); }
	const char *file_name()     { return fileName.c_str(); }
	size_t file_size()          { return length; }
	bool empty()                { return(length == 0); }
	bool open(const char *filename, const char *mode = "r")
	{
		this->close();
		unsigned long mode1, mode2, mode3;
		fileName = std::string(filename);
		if (strcmp(mode, "r") == 0)
		{
			mode1 = GENERIC_READ;
			mode2 = PAGE_READONLY;
			mode3 = FILE_MAP_READ;
		}
		else if (strcmp(mode, "r+") == 0)
		{
			mode1 = GENERIC_READ | GENERIC_WRITE;
			mode2 = PAGE_READWRITE;
			mode3 = FILE_MAP_ALL_ACCESS;
		}
		else
		{
			printf("unknown open mode:%s\n",filename);
			return false;
		}
		
		hFile = CreateFile(filename, mode1, FILE_SHARE_READ, 0, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, 0);
		if(hFile == INVALID_HANDLE_VALUE)
		{
			printf("CreateFile() failed:%s\n",filename);
			return false;
		}
		
		length = GetFileSize(hFile, 0);
		hMap = CreateFileMapping(hFile, 0, mode2, 0, 0, 0);
		if(hMap == NULL)
		{
			printf("CreateFileMapping() failed:%s\n",filename);
			return false;
		}
		
		text = reinterpret_cast<T *>(MapViewOfFile(hMap, mode3, 0, 0, 0));
		if(text == NULL)
		{
			printf("MapViewOfFile() failed:%s\n",filename);
			return false;
		}
		
		return true;
	}
	
	void close()
	{
		if (text)
		{
			UnmapViewOfFile(text);
		}
		
		if (hFile != INVALID_HANDLE_VALUE)
		{
			CloseHandle(hFile);
			hFile = INVALID_HANDLE_VALUE;
		}
		
		if (hMap)
		{
			CloseHandle(hMap);
			hMap = 0;
		}
		
		text = 0;
	}
	
	Mmap(): text(0), hFile(INVALID_HANDLE_VALUE), hMap(0) {}
	virtual ~Mmap() { this->close(); }
};
}

#endif