/*=============================================================================
File:	common.h
Purpose:	implements the string_buffer definition

Special thanks to Taku Kudo <taku@chasen.org> for orginal implementation in
http://crfpp.sourceforge.net/

Simplifies only for readability and understandability
   Copyright(C) langine team
   email: nlptech360@gmail.com
   The license is under GNU GPL

Removes those never used codes in this project
=============================================================================*/

#ifndef CRFPP_NODE_H_
#define CRFPP_NODE_H_

#include <vector>
#include <cmath>
#include "path.h"

namespace CRFPP {

struct Path;
struct Node {
  unsigned int         x;
  unsigned short int   y;
  double               alpha;
  double               beta;
  double               cost;
  double               bestCost;
  Node                *prev;
  int                 *fvector;
  std::vector<Path *>  lpath;
  std::vector<Path *>  rpath;

  void clear() {
    x = y = 0;
    alpha = beta = cost = 0.0;
    prev = 0;
    fvector = 0;
    lpath.clear();
    rpath.clear();
  }

  void shrink() {
    std::vector<Path *>(lpath).swap(lpath);
    std::vector<Path *>(rpath).swap(rpath);
  }

  Node(): x(0), y(0), alpha(0.0), beta(0.0),
          cost(0.0), bestCost(0.0), prev(0), fvector(0) {}
};
}

#endif
