/*=============================================================================
File:	common.h
Purpose:	implements the string_buffer definition

Special thanks to Taku Kudo <taku@chasen.org> for orginal implementation in
http://crfpp.sourceforge.net/

Simplifies only for readability and understandability
   Copyright(C) langine team
   email: nlptech360@gmail.com
   The license is under GNU GPL

Removes those never used codes in this project
=============================================================================*/

#ifndef CRFPP_TAGGER_H_
#define CRFPP_TAGGER_H_

#include <queue>
#include "feature_index.h"

namespace CRFPP {

    static inline double toprob(Node *n, double Z)
	{
        return exp(n->alpha + n->beta - n->cost - Z);
    }

    class TaggerImpl
    {
    private:
        struct QueueElement
		{
            Node *node;
            QueueElement *next;
            double fx;
            double gx;
        };

        class QueueElementComp
		{
        public:
            const bool operator()(QueueElement *q1,
                QueueElement *q2)
            { return(q1->fx > q2->fx); }
        };

        enum { TEST, LEARN };
        unsigned int mode_   : 2;
        unsigned int vlevel_ : 3;
        unsigned int nbest_  : 11;
        size_t                            ysize_;
        double                            cost_;
        double                            Z_;
        size_t                            feature_id_;
        FeatureIndex                     *feature_index_;
        std::vector<std::vector <const char *> > x_;
        std::vector<std::vector <Node *> > node_;
        std::vector<unsigned short int>  answer_;
        std::vector<unsigned short int>  result_;
        string_buffer os_;

        scoped_ptr<std::priority_queue <QueueElement*, std::vector <QueueElement *>,QueueElementComp> > agenda_;
        scoped_ptr<FreeList <QueueElement> > nbest_freelist_;
        void viterbi();
        void buildLattice();
        bool add2(size_t, char* column[]);

    public:
        explicit TaggerImpl(): mode_(TEST), vlevel_(0), nbest_(0),
            ysize_(0), Z_(0), feature_id_(0),
            feature_index_(0) {}
        virtual ~TaggerImpl() { close(); }

        void   set_feature_id(size_t id) { feature_id_  = id; }
        size_t feature_id() const { return feature_id_; }
        Node  *node(size_t i, size_t j) const { return node_[i][j]; }
        void   set_node(Node *n, size_t i, size_t j) { node_[i][j] = n; }
        bool         open(const char* pszModelPath);
        void         close();
        size_t       size() const { return x_.size(); }
        size_t       xsize() const { return feature_index_->xsize(); }
        double       prob() const { return exp(- cost_ - Z_); }
        double       prob(size_t i, size_t j) const
		{
            return toprob(node_[i][j], Z_);
        }
        double       prob(size_t i) const
		{
            return toprob(node_[i][result_[i]], Z_);
        }
        size_t y(size_t i)  const    { return result_[i]; }
        const char* yname(size_t i) const    { return feature_index_->y(i); }
        const char*  x(size_t i, size_t j) const { return x_[i][j]; }
        const char* toString();
        bool clear();
        bool next();
        bool ParseSequence();
        bool AddLine(char* pszLine);
        bool ReadSequence(FILE* fpTest);
        bool ProcessingFile(FILE* fpTest, FILE* fpResult);
    };
}

#endif
